package nl.tudelft.ewi.es.mbdlistener;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.List;

import nl.tudelft.ewi.es.mbdlistener.diagnoser.Diagnoser;
import nl.tudelft.ewi.es.mbdlistener.events.EventList;
import nl.tudelft.ewi.es.mbdlistener.graphics.GuiPrinter;
import nl.tudelft.ewi.es.mbdlistener.graphics.MainPanel;
import nl.tudelft.ewi.es.mbdlistener.input.ListenerInput;
import nl.tudelft.ewi.es.mbdlistener.input.SerialForwarderInput;
import nl.tudelft.ewi.es.mbdlistener.input.XmlInput;
import nl.tudelft.ewi.es.mbdlistener.output.CheckPrinter;
import nl.tudelft.ewi.es.mbdlistener.output.DiagnosesPrinter;
import nl.tudelft.ewi.es.mbdlistener.output.EventPrinter;
import nl.tudelft.ewi.es.mbdlistener.output.ListenerOutput;
import nl.tudelft.ewi.es.mbdlistener.output.PlotPrinter;
import nl.tudelft.ewi.es.mbdlistener.output.TimePrinter;
import nl.tudelft.ewi.es.mbdlistener.output.XmlStreamPrinter;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.GnuParser;
import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;


public class MbdListener { 
	private static boolean startGui = false;

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		List<ListenerInput> inputs = new ArrayList<ListenerInput>();
		List<ListenerOutput> outputs = new ArrayList<ListenerOutput>();
		EventList eventList = new EventList();
		final Diagnoser diagnoser = new Diagnoser();
		// Parse the command line
		Options options = buildOptions();
		parseCommandLine(args, options, inputs, outputs, eventList, diagnoser);

		if (startGui) {
			try {
				javax.swing.SwingUtilities.invokeAndWait(new Runnable() {
					public void run() {
						MainPanel.runGui();
					}
				});
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (InvocationTargetException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			outputs.add(new GuiPrinter(diagnoser, MainPanel.getTreeModel()));
		}
		
		// Add the outputs as observers to the event list
		for (ListenerOutput output:outputs) {
			eventList.addObserver(output);
		}
		
		// Start the inputs
		for (ListenerInput input:inputs) {
			input.start();
		}
	}


	/**
	 * Creates the command line option structure
	 */
	private static Options buildOptions() {
		Options options = new Options();
		options.addOption("help", false, "print this message");

		Option sf = new Option("sf", "use the serialforwarder for input, defaults to localhost:9002");
		sf.setArgName("hostname:port");
		sf.setArgs(2);
		sf.setValueSeparator(':');
		sf.setOptionalArg(true);
		options.addOption(sf);

		options.addOption("ix", "input-xml", false, "Input events as XML");
		Option ixf = new Option("ixf", "output-xml-file", true, "optional filename to output the read from");
		ixf.setArgName("filename");
		options.addOption(ixf);
		options.addOption("ixto", "input-xml-tossim", false, "use TOSSIM timestamps (Default)");
		options.addOption("ixtb", "input-xml-testbed", false, "use testbed timestamps");

		options.addOption("gui", "gui-output", false, "Start the GUI");

		options.addOption("le", "list-events", false, "list all events as they occur");
		Option lef = new Option("lef", "list-events-file", true, "optional filename to output the event list to");
		lef.setArgName("filename");
		options.addOption(lef);
		
		options.addOption("ld", "list-diagnoses", false, "calculate and print the diagnoses after each event");
		Option ldf = new Option("ldf", "list-diagnoses-file", true, "optional filename to output the diagnosis list to");
		ldf.setArgName("filename");
		options.addOption(ldf);

		options.addOption("ce", "check-events", false, "check the output of the diagnoser for false positives and negatives after each event");
		Option cef = new Option("cef", "check-events-file", true, "optional filename to output the list to");
		cef.setArgName("filename");
		options.addOption(cef);

		options.addOption("cp", "check-plot", false, "check the output of the diagnoser for false positives and negatives after each event, and output as data points for gnuplot");
		Option cpf = new Option("cpf", "check-plot-file", true, "optional filename to output the list to");
		cpf.setArgName("filename");
		options.addOption(cpf);

		options.addOption("tp", "time-plot", false, "check the output of the diagnoser for false positives and negatives after each event, and output the time taken to find them for gnuplot");
		Option tpf = new Option("tpf", "time-plot-file", true, "optional filename to output the list to");
		tpf.setArgName("filename");
		options.addOption(tpf);
		
		options.addOption("ox", "output-xml", false, "Output events as XML");
		options.addOption("oxd", "output-xml-diagnoses", false, "output diagnoses to XML as well");
		Option oxf = new Option("oxf", "output-xml-file", true, "optional filename to output the XML to");
		oxf.setArgName("filename");
		options.addOption(oxf);
		return options;
	}

	/**
	 * Parses the command line options, and puts relevant inputs and outputs into the lists.
	 * @param args
	 * @param options
	 * @param inputs
	 * @param eventList 
	 * @param  
	 */
	private static void parseCommandLine(String[] args, Options options, List<ListenerInput> inputs, List<ListenerOutput> outputs, EventList eventList, Diagnoser diagnoser) {
		CommandLineParser parser = new GnuParser();
		try {
			CommandLine line = parser.parse(options, args);
			if (line.hasOption("help")) {
				HelpFormatter formatter = new HelpFormatter();
				formatter.printHelp("mbdListener", options, true);
				System.exit(0);
			}
			
			/*
			 * Input options 
			 */
			
			// Get input from the serial forwarder
			if (line.hasOption("sf")) {
				String values[] = line.getOptionValues("sf");
				String sfhost = "localhost";
				Integer sfport = 9002;

				if(values != null) {
					sfhost = values[0];
					if (values.length == 2) {
						try {
							sfport = Integer.parseInt(values[1]);
						} catch (NumberFormatException e) {
							System.err.println("Invalid port number: " + values[1]);
							System.exit(1);
						}
					}
				}
				inputs.add(new SerialForwarderInput(eventList, sfhost, sfport));
			}

			// Output XML
			if (line.hasOption("ix")) {
				String filename = line.getOptionValue("ixf");
				boolean testbedTime = false;
				
				if (line.hasOption("ixtb"))
					testbedTime = true;
				if (line.hasOption("ixto"))
					testbedTime = false;
				
				if (filename != null) {
					try {
						InputStream is = new FileInputStream(filename);
						inputs.add(new XmlInput(eventList, is, testbedTime));
					} catch (FileNotFoundException e) {
						System.err.println("Error opening output file for --xml-input: " + e.getMessage());
						System.exit(1);
					}
				} else {
					inputs.add(new XmlInput(eventList, System.in, testbedTime));
				}
			}			

			/*
			 * Output options
			 */
		
			if (line.hasOption("gui")) 
				startGui = true;
			// List the diagnoses for each event that comes in, optionally to a file
			if (line.hasOption("ld")) {
				String filename = line.getOptionValue("ldf");
				if (filename != null) {
					try {
						OutputStream os = new FileOutputStream(filename);
						outputs.add(new DiagnosesPrinter(diagnoser, os));
					} catch (FileNotFoundException e) {
						System.err.println("Error opening output file for --list-event: " + e.getMessage());
						System.exit(1);
					}
				} else {
					outputs.add(new DiagnosesPrinter(diagnoser, System.out));
				}
			}
			
			// Just List events, optionally to a file
			if (line.hasOption("le")) {
				String filename = line.getOptionValue("lef");
				if (filename != null) {
					try {
						OutputStream os = new FileOutputStream(filename);
						outputs.add(new EventPrinter(diagnoser, os));
					} catch (FileNotFoundException e) {
						System.err.println("Error opening output file for --list-diagnoses: " + e.getMessage());
						System.exit(1);
					}
				} else {
					outputs.add(new EventPrinter(diagnoser, System.out));
				}
			}

			// Check the events for false positives and negatives
			if (line.hasOption("ce")) {
				String filename = line.getOptionValue("cef");
				if (filename != null) {
					try {
						OutputStream os = new FileOutputStream(filename);
						outputs.add(new CheckPrinter(diagnoser, os));
					} catch (FileNotFoundException e) {
						System.err.println("Error opening output file for --check-events: " + e.getMessage());
						System.exit(1);
					}
				} else {
					outputs.add(new CheckPrinter(diagnoser, System.out));
				}
			}
			
			// Check the events for false positives and negatives
			if (line.hasOption("cp")) {
				String filename = line.getOptionValue("cpf");
				if (filename != null) {
					try {
						OutputStream os = new FileOutputStream(filename);
						outputs.add(new PlotPrinter(diagnoser, os));
					} catch (FileNotFoundException e) {
						System.err.println("Error opening output file for --check-plot: " + e.getMessage());
						System.exit(1);
					}
				} else {
					outputs.add(new PlotPrinter(diagnoser, System.out));
				}
			}			

			// Check the events for false positives and negatives, and output the time they existed
			if (line.hasOption("tp")) {
				String filename = line.getOptionValue("tpf");
				if (filename != null) {
					try {
						OutputStream os = new FileOutputStream(filename);
						outputs.add(new TimePrinter(diagnoser, os));
					} catch (FileNotFoundException e) {
						System.err.println("Error opening output file for --time-plot: " + e.getMessage());
						System.exit(1);
					}
				} else {
					outputs.add(new TimePrinter(diagnoser, System.out));
				}
			}
			
			// Output XML
			if (line.hasOption("ox")) {
				String filename = line.getOptionValue("oxf");
				boolean writeDiagnoses = false;
				if (line.hasOption("oxd"))
					writeDiagnoses = true;
				
				if (filename != null) {
					try {
						OutputStream os = new FileOutputStream(filename);
						outputs.add(new XmlStreamPrinter(diagnoser, os, writeDiagnoses));
					} catch (FileNotFoundException e) {
						System.err.println("Error opening output file for --xml-output: " + e.getMessage());
						System.exit(1);
					}
				} else {
					outputs.add(new XmlStreamPrinter(diagnoser, System.out, writeDiagnoses));
				}
			}			
		} catch (ParseException e) {
			System.err.println("Command line parsing failed. Reason: " + e.getMessage());
			HelpFormatter formatter = new HelpFormatter();
			formatter.printHelp("mbdListener", options);
			System.exit(1);
		}
	}		
}
