package nl.tudelft.ewi.es.mbdlistener.diagnoser;

import java.io.Serializable;
import java.util.Collection;
import java.util.Iterator;
import java.util.SortedSet;
import java.util.TreeSet;

import nl.tudelft.ewi.es.mbdlistener.types.SerialConflictMessage;

public class Conflict implements Serializable, Comparable<Conflict> {
	private static final long serialVersionUID = 1L;
	
	private SortedSet<Environment> conflict = new TreeSet<Environment>();
	private int id;
	private int source;
	private double timestamp;
	
	/**
	 * @param conflict
	 * @param id
	 * @param size
	 * @param source
	 */
	public Conflict(SortedSet<Environment> conflict, int id, int source) {
		this.conflict = conflict;
		this.id = id;
		this.source = source;
	}

	public Conflict(Collection<Environment> conflict, int id, int source) {
		this(new TreeSet<Environment>(conflict), id, source);
	}

	public Conflict(SerialConflictMessage msg) {
		this.id = msg.get_conflict_id();
		this.source = msg.get_source();
		
		short[] fullEnv = msg.get_conflict_conflict();
		int envIndex = 0;
		
		for (int i=0; i<msg.get_conflict_size(); i++) {
			EnvironmentType type = EnvironmentType.byID(fullEnv[envIndex]);
			switch (type) {
			case Link:
				this.conflict.add(new LinkEnvironment(fullEnv, envIndex+1));
				break;
			case Node:
				this.conflict.add(new NodeEnvironment(fullEnv, envIndex+1));
				break;
			case Sensor:
				this.conflict.add(new SensorEnvironment(fullEnv, envIndex+1));
				break;
			case None:
				System.out.println("Invalid environment type: None");
				System.exit(0);
				break;
			}
			envIndex += 1 + type.getSize();
		}
	}
	
	public Conflict(TreeSet<Environment> conflict, int id, int source, double timestamp) {
		this(conflict, id, source);
		this.timestamp = timestamp;
		// TODO Auto-generated constructor stub
	}

	public SortedSet<Environment> getConflict() {
		return conflict;
	}
	
	@Override
	public String toString() {
		String retval = "<";  
		for (Environment env : conflict) {
			retval += env + ", ";
		}
		if(!conflict.isEmpty())
			retval = retval.substring(0, retval.length()-2);
		return retval + "> (Src: " + source + ", ID: " + id + ")";
	}
	@Override
	public boolean equals(Object obj) {
		if (obj instanceof Conflict) {
			Conflict conflictObj = (Conflict) obj;
			return conflict.equals(conflictObj.getConflict());
		}
		return super.equals(obj);
	}
	public int getId() {
		return id;
	}
	public int getSource() {
		return source;
	}
	public int getSize() {
		return this.conflict.size();
	}
	public double getTimestamp() {
		return timestamp;
	}
	@Override
	public int compareTo(Conflict o) {
		if (o.getConflict().size() > conflict.size())
			return -1;
		if (o.getConflict().size() < conflict.size())
			return 1;
		
		Iterator<Environment> thisIterator = conflict.iterator();
		Iterator<Environment> otherIterator = o.getConflict().iterator();
		
		for (int i=0; i<conflict.size(); i++) {
			int val = thisIterator.next().compareTo(otherIterator.next());
			if (val != 0)
				return val;
		}
		
		return 0;
	}
}
