package nl.tudelft.ewi.es.mbdlistener.diagnoser;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.Observable;
import java.util.Set;
import java.util.TreeSet;
import java.util.Map.Entry;

import nl.tudelft.ewi.es.mbdlistener.types.Tuple;



public class ConflictSet extends Observable {
	Map<Tuple<Integer,Integer>, Conflict> conflicts = new HashMap<Tuple<Integer,Integer>, Conflict>();
	Set<Conflict> conflictSet = new TreeSet<Conflict> ();
	
	public void put(Tuple<Integer, Integer> id, Conflict conflict) {
		conflicts.put(id, conflict);
		conflictSet.add(conflict);
		
		setChanged();
		notifyObservers(conflict);
	}
	
	public Conflict get(Tuple<Integer, Integer> id) {
		return conflicts.get(id);
	}

	public Conflict remove(Tuple<Integer, Integer> id) {
		setChanged();
		notifyObservers(id);

		Conflict removable = conflicts.remove(id); 
		if (removable != null && conflictSet.contains(removable)) {
			TreeSet<Conflict> newConflictSet = new TreeSet<Conflict>(conflicts.values());
			if (!newConflictSet.equals(conflictSet)) {
				conflictSet = newConflictSet;
				setChanged();
				notifyObservers(conflictSet);
			}
		}
		
		return removable;
	}

	public int size() {
		return conflicts.size();
	}
	
	public Collection<Conflict> values() {
		return conflicts.values();
	}
	

	@Override
	public String toString() {
		String retval = "";
		for (Entry<Tuple<Integer, Integer>, Conflict> entry : conflicts.entrySet()) {
			retval += "Conflict from " +entry.getKey().var1() + ", id "+ entry.getKey().var1() +": " + entry.getValue() + ", ";
		}
		if (!conflicts.isEmpty())
			retval = retval.substring(0, retval.length()-2);
		return retval;
	}

	public void clear() {
		conflicts.clear();
		conflictSet.clear();
	}
}
