package nl.tudelft.ewi.es.mbdlistener.graphics;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GridLayout;
import java.awt.RenderingHints;
import java.awt.geom.AffineTransform;
import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JPanel;

import nl.tudelft.ewi.es.mbdlistener.diagnoser.Diagnosis;
import nl.tudelft.ewi.es.mbdlistener.diagnoser.Environment;
import nl.tudelft.ewi.es.mbdlistener.diagnoser.LinkEnvironment;
import nl.tudelft.ewi.es.mbdlistener.diagnoser.NodeEnvironment;
import nl.tudelft.ewi.es.mbdlistener.diagnoser.SensorEnvironment;

public class GraphPane extends JPanel {
	private static final long serialVersionUID = 1L;
	private List<Node> nodes = new ArrayList<Node>(); 
	private Diagnosis diagnosis = null;
	
	private float maxX, maxY;
	
	public GraphPane() {
		super(new GridLayout(0,1));
	    setBackground(Color.WHITE);

	    try {
    	    FileInputStream fstream = new FileInputStream("positions.trans");
//    	    FileInputStream fstream = new FileInputStream("topology.out");
    	    BufferedReader breader = new BufferedReader(new InputStreamReader(fstream));
    	    String line;
    	    while ((line = breader.readLine()) != null) {
    	    	String comps[] = line.split("\t");
    	    	float y = Float.parseFloat(comps[2]);
				float x = Float.parseFloat(comps[1]);
				
				maxX = Math.max(x, maxX);
				maxY = Math.max(y, maxY);
				
				Node node = new Node(Integer.parseInt(comps[0]), x, y);
    	    	nodes.add(node);
    	    }
		} catch (FileNotFoundException e) {
			throw new RuntimeException(e);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public void setDiagnosis(Diagnosis diagnosis) {
		this.diagnosis = diagnosis;
		for (Node node: nodes) {
			node.setFaulty(false);
			node.setSensorFaulty(false);
			for (Environment env : diagnosis.getDiagnosis()) {
				if (env instanceof NodeEnvironment) { 
					if (node.getNodeId() == ((NodeEnvironment)env).getNodeID())
						node.setFaulty(true);
				}
				if (env instanceof SensorEnvironment) { 
					if (node.getNodeId() == ((SensorEnvironment)env).getNodeID())
						node.setSensorFaulty(true);
				}
			}
		}
		this.repaint();
	}
	
	@Override
	protected void paintComponent(Graphics g) {
		super.paintComponent(g);
		Graphics2D g2 = (Graphics2D)g;
        g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);		
		
        double xScale = getBounds().width/ (maxX * 1.1);
        double yScale = getBounds().height/ (maxY * 1.1);
        
        double scale = Math.min(xScale, yScale);
        
        g2.transform(AffineTransform.getTranslateInstance(0, 20));
		g2.transform(AffineTransform.getScaleInstance(scale, scale));
		
		if (diagnosis != null) {
			for (Environment env : diagnosis.getDiagnosis()) {
				if (env instanceof LinkEnvironment) {
					double receiverX = 0;
					double receiverY = 0;
					double senderX = 0;
					double senderY = 0;
					for (Node node: nodes) {
						if (node.getNodeId() == ((LinkEnvironment)env).getReceiver()) {
							receiverX = node.getX();
							receiverY = node.getY();
						}
						if (node.getNodeId() == ((LinkEnvironment)env).getSender()) {
							senderX = node.getX();
							senderY = node.getY();
						}
					}
					g2.drawLine((int)receiverX, (int)receiverY, (int)senderX, (int)senderY);
				}
			}
		}
		for (Node node : nodes) {
			node.paintNode(g2, scale);
		}
		
		g2.transform(AffineTransform.getScaleInstance(1, 1));
	}
}
