package nl.tudelft.ewi.es.mbdlistener.input;

import java.io.IOException;
import java.util.Calendar;

import net.tinyos.message.Message;
import net.tinyos.message.MessageListener;
import net.tinyos.message.MoteIF;
import net.tinyos.packet.BuildSource;
import net.tinyos.packet.PhoenixError;
import net.tinyos.packet.PhoenixSource;
import net.tinyos.util.Messenger;
import nl.tudelft.ewi.es.mbdlistener.diagnoser.Conflict;
import nl.tudelft.ewi.es.mbdlistener.events.ConflictCancellationEvent;
import nl.tudelft.ewi.es.mbdlistener.events.ConflictEvent;
import nl.tudelft.ewi.es.mbdlistener.events.EventList;
import nl.tudelft.ewi.es.mbdlistener.events.MessageEvent;
import nl.tudelft.ewi.es.mbdlistener.events.NodeDeathEvent;
import nl.tudelft.ewi.es.mbdlistener.events.SensorDeathEvent;
import nl.tudelft.ewi.es.mbdlistener.types.DebugMessage;
import nl.tudelft.ewi.es.mbdlistener.types.SerialConflictMessage;

public class SerialForwarderInput extends ListenerInput implements PhoenixError, Messenger, MessageListener{
	private String sfhost;
	private Integer sfport;
	private MoteIF mote;
	private PhoenixSource source;
	private double startTimeStamp;
	
	public SerialForwarderInput(EventList eventList, String sfhost, Integer sfport) {
		super(eventList);
		
		this.sfhost = sfhost;
		this.sfport = sfport;

		// Get ugly source (Do I have to?)
		source = BuildSource.makePhoenix("sf@" + sfhost + ":" + sfport, this);
		
		// Prevent it from using System.exit() (!)
		source.setResurrection();
		source.setPacketErrorHandler(this);
	}

	@Override
	public void start() {
		startTimeStamp = Calendar.getInstance().getTimeInMillis();
		source.start();

		// Subscribe to SerialConflict messages 
		mote = new MoteIF(source);
		mote.registerListener(new SerialConflictMessage(), this);
		mote.registerListener(new DebugMessage(), this);
	}
	// Called on incoming message
	@Override
	public void messageReceived(int to, Message m) {
		Double timeStamp = (Calendar.getInstance().getTimeInMillis() - startTimeStamp) / 1000.0;
		
		if (m instanceof SerialConflictMessage) {
			SerialConflictMessage conflictMsg = (SerialConflictMessage) m;

			// Get sensor death events
			if (to == 65280) {
				eventList.add(new SensorDeathEvent(timeStamp, conflictMsg.get_source()));
				return;
			}
			
			// Get conflict cancellations
			if (conflictMsg.get_conflict_size() == 0) {
				eventList.add(new ConflictCancellationEvent(timeStamp, conflictMsg.get_source(), conflictMsg.get_conflict_id()));
				return;
			}
			
			Conflict conflict = new Conflict(conflictMsg);
			eventList.add(new ConflictEvent(timeStamp, conflict));
			return;
		}
		if (m instanceof DebugMessage) {
			DebugMessage debugMsg = (DebugMessage) m;
			switch (debugMsg.get_type()) {
				case 1:
					eventList.add(new NodeDeathEvent(timeStamp, (int)debugMsg.get_data4()));
					break;
				case 7:
					eventList.add(new MessageEvent(timeStamp, (int) debugMsg.get_data4()));
			}
		}
	}

	@Override
	public void message(String s) {
		System.err.println(s);
	}

	@Override
	public void error(IOException e) {
		System.err.println("Serialforwarder could not connect to " + sfhost + ":" + sfport + ", retrying...");
		try {
			Thread.sleep(1000);
		} catch (InterruptedException e1) {
		}
	}

}
