package nl.tudelft.ewi.es.mbdlistener.output;

import java.io.OutputStream;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Observable;

import nl.tudelft.ewi.es.mbdlistener.diagnoser.Diagnoser;
import nl.tudelft.ewi.es.mbdlistener.diagnoser.Diagnosis;
import nl.tudelft.ewi.es.mbdlistener.diagnoser.Environment;
import nl.tudelft.ewi.es.mbdlistener.diagnoser.NodeEnvironment;
import nl.tudelft.ewi.es.mbdlistener.diagnoser.SensorEnvironment;
import nl.tudelft.ewi.es.mbdlistener.events.ConflictCancellationEvent;
import nl.tudelft.ewi.es.mbdlistener.events.ConflictEvent;
import nl.tudelft.ewi.es.mbdlistener.events.NodeDeathEvent;
import nl.tudelft.ewi.es.mbdlistener.events.SensorDeathEvent;

public class CheckPrinter extends ListenerOutput {
	private final PrintStream outputStream;
	private List<Integer> deadNodes = new ArrayList<Integer>();
	private List<Integer> deadSensors = new ArrayList<Integer>();

	public CheckPrinter(Diagnoser diagnoser, OutputStream os) {
		super(diagnoser);
		this.outputStream = new PrintStream(os, true);
	}
	
	// Print messages to console. 
	@Override
	public void update(Observable o, Object arg) {
		super.update(o,arg);
		
		if (arg instanceof NodeDeathEvent) {
			deadNodes.add(((NodeDeathEvent) arg).getNodeId());
			checkConflicts();
		}
		if (arg instanceof SensorDeathEvent) {
			deadSensors.add(((SensorDeathEvent) arg).getNodeId());
			checkConflicts();
		}
		if (arg instanceof ConflictEvent || arg instanceof ConflictCancellationEvent) {
			checkConflicts();
		}
	}

	private void checkConflicts() {
		List<Integer> nodeFalsePositives = new ArrayList<Integer>();
		List<Integer> sensorFalsePositives = new ArrayList<Integer>();
		List<Integer> nodeMissed = new ArrayList<Integer>(deadNodes);
		List<Integer> sensorMissed = new ArrayList<Integer>(deadSensors);

		Diagnosis bestDiagnosis = getBestDiagnosis();
		for (Environment env : bestDiagnosis.getDiagnosis()) {
			if (env instanceof NodeEnvironment) {
				Integer nodeID = ((NodeEnvironment) env).getNodeID();
				if (!deadNodes.contains(nodeID)) {
					nodeFalsePositives.add(nodeID);
				}
				nodeMissed.remove(nodeID);
			} else if (env instanceof SensorEnvironment) {
				Integer nodeID = ((SensorEnvironment) env).getNodeID();
				if (!deadSensors.contains(nodeID)) {
					sensorFalsePositives.add(nodeID);
				}
				sensorMissed.remove(nodeID);
			}
		}
		
		outputStream.println("\tBest diagnosis: " + bestDiagnosis);
		outputStream.println("\tDead: Sensor: " + deadSensors + ", Node: " + deadNodes);
		outputStream.println("\tFalse positives: Sensor: " + sensorFalsePositives + ", Node: " + nodeFalsePositives);
		outputStream.println("\tMissed: Sensor: " + sensorMissed + ", Node: " + nodeMissed);
	}
}
