package nl.tudelft.ewi.es.mbdlistener.output;

import java.io.OutputStream;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Observable;

import nl.tudelft.ewi.es.mbdlistener.diagnoser.Diagnoser;
import nl.tudelft.ewi.es.mbdlistener.diagnoser.Diagnosis;
import nl.tudelft.ewi.es.mbdlistener.diagnoser.Environment;
import nl.tudelft.ewi.es.mbdlistener.diagnoser.NodeEnvironment;
import nl.tudelft.ewi.es.mbdlistener.diagnoser.SensorEnvironment;
import nl.tudelft.ewi.es.mbdlistener.events.ConflictCancellationEvent;
import nl.tudelft.ewi.es.mbdlistener.events.ConflictEvent;
import nl.tudelft.ewi.es.mbdlistener.events.ListenerEvent;
import nl.tudelft.ewi.es.mbdlistener.events.NodeDeathEvent;
import nl.tudelft.ewi.es.mbdlistener.events.SensorDeathEvent;

public class PlotPrinter extends ListenerOutput {
	private final PrintStream outputStream;
	private List<Integer> deadNodes = new ArrayList<Integer>();
	private List<Integer> deadSensors = new ArrayList<Integer>();
	private List<Integer> previousNodeMissed = new ArrayList<Integer>();
	private List<Integer> previousSensorMissed = new ArrayList<Integer>();

	public PlotPrinter(Diagnoser diagnoser, OutputStream os) {
		super(diagnoser);
		this.outputStream = new PrintStream(os, true);
	}
	
	// Print messages to console. 
	@Override
	public void update(Observable o, Object arg) {
		super.update(o,arg);
		
		if (arg instanceof NodeDeathEvent) {
			NodeDeathEvent nodeDeathEvent = (NodeDeathEvent) arg;
			deadNodes.add(nodeDeathEvent.getNodeId());
			checkConflicts(nodeDeathEvent, 1);
		}
		if (arg instanceof SensorDeathEvent) {
			SensorDeathEvent sensorDeathEvent = (SensorDeathEvent) arg;
			deadSensors.add(sensorDeathEvent.getNodeId());
			checkConflicts(sensorDeathEvent, 0);
		}
		if (arg instanceof ConflictEvent || arg instanceof ConflictCancellationEvent) {
			checkConflicts((ListenerEvent)arg, 2);
		}
	}

	private void checkConflicts(ListenerEvent event, int column) {
		List<Integer> nodeFalsePositives = new ArrayList<Integer>();
		List<Integer> sensorFalsePositives = new ArrayList<Integer>();
		List<Integer> nodeMissed = new ArrayList<Integer>(deadNodes);
		List<Integer> sensorMissed = new ArrayList<Integer>(deadSensors);

		Diagnosis bestDiagnosis = getBestDiagnosis();
		for (Environment env : bestDiagnosis.getDiagnosis()) {
			if (env instanceof NodeEnvironment) {
				Integer nodeID = ((NodeEnvironment) env).getNodeID();
				if (!deadNodes.contains(nodeID)) {
					nodeFalsePositives.add(nodeID);
				}
				nodeMissed.remove(nodeID);
			} else if (env instanceof SensorEnvironment) {
				Integer nodeID = ((SensorEnvironment) env).getNodeID();
				if (!deadSensors.contains(nodeID)) {
					sensorFalsePositives.add(nodeID);
				}
				sensorMissed.remove(nodeID);
			}
		}
		outputStream.print ("" + event.getTimeStamp() + " " + sensorFalsePositives.size() + " " + nodeFalsePositives.size() + " " + sensorMissed.size() + " " + nodeMissed.size());
		
		for (int i=0; i<3; i++) {
			if (column == i)
				System.out.print(" \"" + event + "\"");
			else
				System.out.print(" \"\"");
		}
		
		if (previousSensorMissed.equals(sensorMissed))
			System.out.print (" \"\"");
		else
			System.out.print (" \""+ sensorMissed +"\"");
		if (previousNodeMissed.equals(nodeMissed))
			System.out.print (" \"\"");
		else
			System.out.print (" \""+ nodeMissed +"\"");
		System.out.println();
		
		previousNodeMissed = nodeMissed;
		previousSensorMissed = sensorMissed;		
	}
}
