package nl.tudelft.ewi.es.mbdlistener.output;

import java.io.OutputStream;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Observable;

import nl.tudelft.ewi.es.mbdlistener.diagnoser.Diagnoser;
import nl.tudelft.ewi.es.mbdlistener.diagnoser.Diagnosis;
import nl.tudelft.ewi.es.mbdlistener.diagnoser.Environment;
import nl.tudelft.ewi.es.mbdlistener.diagnoser.NodeEnvironment;
import nl.tudelft.ewi.es.mbdlistener.diagnoser.SensorEnvironment;
import nl.tudelft.ewi.es.mbdlistener.events.ConflictCancellationEvent;
import nl.tudelft.ewi.es.mbdlistener.events.ConflictEvent;
import nl.tudelft.ewi.es.mbdlistener.events.EndEvent;
import nl.tudelft.ewi.es.mbdlistener.events.ListenerEvent;
import nl.tudelft.ewi.es.mbdlistener.events.NodeDeathEvent;
import nl.tudelft.ewi.es.mbdlistener.events.SensorDeathEvent;

public class TimePrinter extends ListenerOutput {
	private final PrintStream outputStream;
	private List<Integer> deadNodes = new ArrayList<Integer>();
	private List<Integer> deadSensors = new ArrayList<Integer>();
	private List<Integer> previousNodeMissed = new ArrayList<Integer>();
	private List<Integer> previousSensorMissed = new ArrayList<Integer>();
	private HashMap<Integer,Double> nodeDeathTime = new HashMap<Integer, Double>();
	private HashMap<Integer,Double> nodeFoundTime = new HashMap<Integer, Double>();
	private HashMap<Integer,Double> sensorDeathTime = new HashMap<Integer, Double>();
	private HashMap<Integer,Double> sensorFoundTime = new HashMap<Integer, Double>();
	private HashMap<Integer,Double> nodeFalseTime1 = new HashMap<Integer, Double>();
	private HashMap<Integer,Double> sensorFalseTime1 = new HashMap<Integer, Double>();
	private List<Integer> previousSensorFalse = new ArrayList<Integer>();
	private List<Integer> previousNodeFalse = new ArrayList<Integer>();
	
	public TimePrinter(Diagnoser diagnoser, OutputStream os) {
		super(diagnoser);
		this.outputStream = new PrintStream(os, true);
	}
	
	// Print messages to console. 
	@Override
	public void update(Observable o, Object arg) {
		super.update(o,arg);
		
		if (arg instanceof NodeDeathEvent) {
			NodeDeathEvent nodeDeathEvent = (NodeDeathEvent) arg;
			deadNodes.add(nodeDeathEvent.getNodeId());
			nodeDeathTime.put(nodeDeathEvent.getNodeId(), nodeDeathEvent.getTimeStamp());
		}
		if (arg instanceof SensorDeathEvent) {
			SensorDeathEvent sensorDeathEvent = (SensorDeathEvent) arg;
			deadSensors.add(sensorDeathEvent.getNodeId());
			sensorDeathTime.put(sensorDeathEvent.getNodeId(), sensorDeathEvent.getTimeStamp());
		}
		if (arg instanceof ConflictEvent || arg instanceof ConflictCancellationEvent) {
			checkConflicts((ListenerEvent)arg, 2);
		}
		if (arg instanceof EndEvent) {
			for (int node: nodeDeathTime.keySet()){ 
				if (!nodeFoundTime.containsKey(node)) {
					if (previousNodeMissed.contains(node)) {
						outputStream.println("\"\" " + "inf"  + " \"\" " + " \"\" " + " \"\" " + node );
					}
				}
			}
			for (int sensor: sensorDeathTime.keySet()){ 
				if (!sensorFoundTime.containsKey(sensor)) {
					if (previousSensorMissed.contains(sensor)) {
						outputStream.println("inf" + " " + "\"\" "  + " \"\" " + " \"\" " +sensor + " \"\" ");
					}
				}
			}
			for (int node: nodeFalseTime1.keySet()){ 
				outputStream.println("\"\" " + " \"\" " + " \"\" " +  "inf" + " \"\" " + node );
			}
			for (int sensor: sensorFalseTime1.keySet()){ 
				outputStream.println("\"\" " + " \"\" " + "inf" + " \"\" " +sensor + " \"\" ");
			}				
		}
	}

	private void checkConflicts(ListenerEvent event, int column) {
		List<Integer> nodeFalsePositives = new ArrayList<Integer>();
		List<Integer> sensorFalsePositives = new ArrayList<Integer>();
		List<Integer> nodeMissed = new ArrayList<Integer>(deadNodes);
		List<Integer> sensorMissed = new ArrayList<Integer>(deadSensors);

		Diagnosis bestDiagnosis = getBestDiagnosis();
		for (Environment env : bestDiagnosis.getDiagnosis()) {
			if (env instanceof NodeEnvironment) {
				Integer nodeID = ((NodeEnvironment) env).getNodeID();
				if (!deadNodes.contains(nodeID)) {
					nodeFalsePositives.add(nodeID);
				}
				nodeMissed.remove(nodeID);
			} else if (env instanceof SensorEnvironment) {
				Integer nodeID = ((SensorEnvironment) env).getNodeID();
				if (!deadSensors.contains(nodeID)) {
					sensorFalsePositives.add(nodeID);
				}
				sensorMissed.remove(nodeID);
			}
		}
		for (int node: nodeDeathTime.keySet()){ 
			if (!nodeFoundTime.containsKey(node)) {
				if (!nodeMissed.contains(node)) {
					nodeFoundTime.put(node, event.getTimeStamp());
					outputStream.println("\"\" " + (event.getTimeStamp() - nodeDeathTime.get(node)) +" \"\" " +" \"\" " +  " \"\" " + node );
				}
			}
		}
		for (int sensor: sensorDeathTime.keySet()){ 
			if (!sensorFoundTime.containsKey(sensor)) {
				if (!sensorMissed.contains(sensor)) {
					sensorFoundTime.put(sensor, event.getTimeStamp());
					outputStream.println(event.getTimeStamp() - sensorDeathTime.get(sensor) + " " + "\"\" " +" \"\" " +" \"\" " + sensor + " \"\" ");
				}
			}
		}
		for (int node: previousNodeFalse){ 
			if (!nodeFalsePositives.contains(node)) {
				if (nodeFalseTime1.containsKey(node)) {
					outputStream.println("\"\" " +" \"\" " +" \"\" " + (event.getTimeStamp() - nodeFalseTime1.remove(node)) + " \"\" " + node );
					nodeFalseTime1.remove(node);
				} else {
					throw new RuntimeException("Error1");
				}
			}
		}
		for (int node: nodeFalsePositives){ 
			if (!previousNodeFalse.contains(node)) {
				if (!nodeFalseTime1.containsKey(node)) {
					nodeFalseTime1.put(node, event.getTimeStamp());
				} else {
					throw new RuntimeException("Error1");
				}
			}
		}
		for (int sensor: previousSensorFalse){ 
			if (!sensorFalsePositives.contains(sensor)) {
				if (sensorFalseTime1.containsKey(sensor)) {
					outputStream.println("\"\" " +" \"\" " + (event.getTimeStamp() - sensorFalseTime1.remove(sensor)) +" \"\" " + " \"\" " + sensor );
					sensorFalseTime1.remove(sensor);
				} else {
					outputStream.println(sensorFalseTime1);
					throw new RuntimeException("Error3");
				}
			}
		}
		for (int sensor: sensorFalsePositives){ 
			if (!previousSensorFalse.contains(sensor)) {
				if (!sensorFalseTime1.containsKey(sensor)) {
					sensorFalseTime1.put(sensor, event.getTimeStamp());
				} else {
					throw new RuntimeException("Error4");
				}
			}
		}
		previousNodeMissed = nodeMissed;
		previousSensorMissed = sensorMissed;
		previousNodeFalse = nodeFalsePositives;
		previousSensorFalse = sensorFalsePositives;		
		previousNodeMissed = nodeMissed;
		previousSensorMissed = sensorMissed;		
	}
}
