/*
 * Copyright (c) 2006 Intel Corporation
 * All rights reserved.
 *
 * This file is distributed under the terms in the attached INTEL-LICENSE
 * file. If you do not find these files, copies can be found by writing to
 * Intel Research Berkeley, 2150 Shattuck Avenue, Suite 1300, Berkeley, CA,
 * 94704.  Attention:  Intel License Inquiry.
 */
//#include "Ctp.h"

/**
 * @author David Gay
 * @author Kyle Jamieson
 */

#ifdef TOSSIM
//#define DEBUG_INFERENCE
//#define DEBUG_ADD_OBSERVATION
//#define DEBUG_ADD_RULE
#endif

#ifndef TESTMBD_H
#define TESTMBD_H
#define TOS_VERSION_21

enum {
  /* Number of readings per message. If you increase this, you may have to
     increase the message_t size. */
    NREADINGS = 5,
  /* Default sampling period. */
    AM_TESTMBD = 0x44,
    AM_CONFLICT = 0x45,
    AM_SERIAL_CONFLICT = 0x46,
    AM_SERIAL_DEBUG = 0x47,
//  MAX_CONFLICT_SIZE = TOSH_DATA_LENGTH-sizeof(ctp_data_header_t)-2
    MAX_CONFLICT_SIZE = 15,

    // If route quality >= this threshold, rules will not be added to the model for this neighbour.
    ROUTE_QUALITY_THRESHOLD = 500,
    // If link quality >= this this threshold, rules will not be added to the model for this neighbour. 255 is the maximum, and used for bad links.
    LINK_QUALITY_THRESHOLD = 255,
    // etx Threshold
    ETX_QUALITY_THRESHOLD = 600,
    // The size of the table where reception of neighbouring beacon messages are kept.
    NEIGHBOUR_TABLE_SIZE = 8,
};

// When should the (static) model be created?
const uint32_t MODEL_CREATION_TIME = 500l * 1024;
// How often should we sample the sensor.
const uint32_t DEFAULT_INTERVAL = 10l * 1024;

// Should we check link, route, etx quality?
#define CHECK_LINK_QUALITY
// Uncomment to disable the serial port on non-root nodes.
//#define SERIAL_ONLY_ON_ROOT

// How often should we check for beacon messages received from our neighbour. Should be at least CTP's maxInterval.
const uint32_t BEACON_CHECK_INTERVAL = 400l *1024;

// When to kill a node
const uint8_t NODE_DEATH_NUMBER = 3;
const uint16_t NODE_DEATH_ID[3] = {6, 31, 48};
const uint32_t NODE_DEATH_TIME[3] = {1200l * 1024, 1500l * 1024, 2100l * 1024};

// Start killing sensors at this time, 0 for no deaths
const uint32_t SENSOR_DEATH_START = 600l * 1024;
// Time between sensor death (DEATH_START + DEATH_INTERVAL * nodeid)
const uint32_t SENSOR_DEATH_INTERVAL = 50l *1024;
// Which sensors to kill (sensornumber % DENSITY == 0 get killed)
const uint32_t SENSOR_DEATH_DENSITY = 10l;

enum {
	DBG_NODE_DEATH = 1,
	DBG_NO_RULES,
	DBG_ROUTE_QUALITY,
	DBG_BAD_ROUTE_QUALITY,
	DBG_SENT_BEACON,
	DBG_ADD_RULE_FAIL,
	DBG_CONFLICT_MESSAGE,
	DBG_CONFLICT,
	DBG_CANCEL_CONFLICT,
};

typedef nx_struct testmbd {
  nx_uint16_t version; /* Version of the interval. */
  nx_uint16_t interval; /* Samping period. */
  nx_uint16_t id; /* Mote id of sending mote. */
  nx_uint16_t count; /* The readings are samples count * NREADINGS onwards */
  nx_uint16_t readings[NREADINGS];
} testmbd_t;

typedef nx_struct conflict {
  nx_uint8_t size; /* Size of the conflict. */
  nx_uint8_t id; /* ID of the conflict. */
  nx_uint8_t conflict[MAX_CONFLICT_SIZE];
} conflict_t;

typedef nx_struct serial_conflict {
  nx_uint16_t source; /* Source of the conflict. */
  conflict_t conflict;
} serial_conflict_t;

typedef nx_struct serial_debug {
  nx_uint16_t type; /* Source of the conflict. */
  nx_uint32_t data1;
  nx_uint32_t data2;
  nx_uint32_t data3;
  nx_uint32_t data4;
} serial_debug_t;

typedef struct neighbour_table {
//	am_addr_t node_id;
	uint16_t node_id;
	bool received_beacon;
} neighbour_table_t;

#endif
